import * as t from 'drizzle-orm/pg-core';
import { timestamps } from '../util';
import { eq, relations } from 'drizzle-orm';
import { users } from './users';
import { breakperiods } from './breakperiods';
import { fields } from './fields';
import { teams } from './teams';
import { rounds } from './rounds';
import { db } from '..';

export const competitions = t.pgTable('competitions', {
	id: t.serial('id').primaryKey(),
	name: t.text('name').notNull(),
	description: t.text('description'),
	start_date: t.timestamp({ withTimezone: true, mode: 'date' }).notNull(),
	location: t.text('location'),
	owner: t.text('owner').notNull(),
	...timestamps
});

export const competitionsRelations = relations(competitions, ({ one, many }) => ({
	owner: one(users, {
		fields: [competitions.owner],
		references: [users.id]
	}),
	breakperiods: many(breakperiods),
	fields: many(fields),
	teams: many(teams),
	rounds: many(rounds)
}));

export type Competition = typeof competitions.$inferSelect;

export async function getCompetitionWithAll(id: number) {
	return await db.query.competitions.findFirst({
		where: eq(competitions.id, id),
		with: {
			breakperiods: true,
			fields: true,
			rounds: {
				with: {
					matches: true,
					schedulingMode: true
				}
			},
			teams: true
		}
	});
}

export async function getCompetitionsWithAll(skip: number = 0, take: number = 10) {
	return await db.query.competitions.findMany({
		orderBy: (competitions, { desc }) => [desc(competitions.start_date)],
		with: {
			breakperiods: true,
			fields: true,
			rounds: {
				with: {
					matches: true,
					schedulingMode: true
				}
			},
			teams: true
		},
		limit: take,
		offset: skip
	});
}
