import * as t from 'drizzle-orm/pg-core';
import { timestamps } from '../util';
import { relations } from 'drizzle-orm';
import { fields } from './fields';
import { rounds } from './rounds';

export const status = t.pgEnum('match_status', ['pending', 'running', 'finished', 'cancelled']);

export const matches = t.pgTable('matches', {
	id: t.serial('id').primaryKey(),
	field_id: t.integer('field_id').notNull(),
	team1: t.text('team1').notNull(),
	team2: t.text('team2').notNull(),
	start_date: t.timestamp({ withTimezone: true, mode: 'date' }).notNull(),
	end_date: t.timestamp({ withTimezone: true, mode: 'date' }).notNull(),
	score1: t.integer('score1').notNull(),
	score2: t.integer('score2').notNull(),
	status: status('status').notNull(),
	match_number: t.integer('match_number').notNull(),
	round_id: t.integer('round').notNull(),
	...timestamps
});

export const matchRelations = relations(matches, ({ one }) => ({
	field: one(fields, {
		fields: [matches.field_id],
		references: [fields.id]
	}),
	round: one(rounds, {
		fields: [matches.round_id],
		references: [rounds.id]
	})
}));

export type Match = typeof matches.$inferSelect;
