import { fail, redirect } from '@sveltejs/kit';
import { db } from '$lib/server/db';
import { competitions } from '$lib/server/db/schema/competitions';
import type { Actions } from './$types';

export const actions = {
	default: async (event) => {
		const data = await event.request.formData();

		console.log('data', data);
		const name = data.get('name')?.toString();
		const description = data.get('description')?.toString() || null;
		const startDateStr = data.get('start_date')?.toString();
		const location = data.get('location')?.toString() || null;

		// Validate required fields
		if (!name) {
			console.log('name', name);
			return fail(400, { error: 'Competition name is required' });
		}

		if (!startDateStr) {
			console.log('startDateStr', startDateStr);
			return fail(400, { error: 'Start date is required' });
		}

		let startDate: Date;
		try {
			startDate = new Date(startDateStr);
			if (isNaN(startDate.getTime())) {
				console.log('startDate', startDate);
				throw new Error('Invalid date');
			}
		} catch {
			console.log('invalid start date format');
			return fail(400, { error: 'Invalid start date format' });
		}

		// Create the competition
		const [competition] = await db
			.insert(competitions)
			.values({
				name,
				description,
				start_date: startDate,
				location,
				owner: event.locals.user.id,
				created_at: new Date(),
				updated_at: new Date()
			})
			.returning();

		throw redirect(303, `/competitions/${competition.id}`);
	}
} satisfies Actions;
